import 'package:flutter/material.dart';

class PickerWidget extends StatefulWidget {
  PickerWidget({
    Key key,
    this.list = const [],
    this.height = 60.0,
    this.labelAlias = 'label',
    this.valueAlias = 'value',
    this.defaultIndex = 0,
    this.onChanged,
  }) : super(key: key);

  final List list;
  final double height;
  final String labelAlias;
  final String valueAlias;
  final int defaultIndex;
  final Function onChanged;

  _PickerWidget createState() => new _PickerWidget();
}

class _PickerWidget extends State<PickerWidget>
    with SingleTickerProviderStateMixin {
  AnimationController _overallMoveController;
  ScrollController _controller;

  int get _length {
    return widget.list.length;
  }

  double get _height {
    return widget.height;
  }

  double get scrollHeight {
    return (_length + 4) * _height;
  }

  @override
  void initState() {
    super.initState();
    double value = widget.defaultIndex * 1.0;
    _overallMoveController = AnimationController(
      vsync: this,
      value: value * _height / scrollHeight,
      duration: const Duration(milliseconds: 200),
    );
    _controller = ScrollController(initialScrollOffset: value * _height);
  }

  @override
  dispose() {
    _overallMoveController.dispose();
    _controller.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant PickerWidget oldWidget) {
    if (oldWidget.list.length != widget.list.length ||
        oldWidget.height != widget.height) {
      if (_controller.offset != 0) {
        _controller.jumpTo(0);
      }
    }
    super.didUpdateWidget(oldWidget);
  }

  void _handleDragEnd(double details) {
    int scrollIndex = (details * (_length + 4)).round().clamp(0.0, _length);
    double scroll = scrollIndex * _height;
    if (_controller.offset != scroll) {
      double scrollRatio = (scroll / ((_length + 4) * _height));
      _overallMoveController
          .animateTo(
        scrollRatio,
        curve: Curves.easeIn,
        duration: const Duration(milliseconds: 200),
      )
          .then((value) {
        if (widget.onChanged != null) {
          widget.onChanged(scrollIndex);
        }
        _controller.jumpTo(scroll);
      });
    } else {
      if (widget.onChanged != null) {
        widget.onChanged(scrollIndex);
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: _height * 5,
      color: Colors.white,
      child: Stack(
        children: [
          Positioned(
            left: 0,
            right: 0,
            top: 0,
            child: SlideTransition(
              position: Tween<Offset>(
                begin: Offset.zero,
                end: Offset(0.0, -1.0),
              ).animate(_overallMoveController),
              child: FractionallySizedBox(
                widthFactor: 1,
                child: Column(
                  children: [
                    Container(
                      height: _height,
                      color: Colors.transparent,
                      alignment: Alignment.center,
                    ),
                    Container(
                      height: _height,
                      color: Colors.transparent,
                      alignment: Alignment.center,
                    ),
                    ...widget.list
                        .map(
                          (e) => Container(
                            height: _height,
                            alignment: Alignment.center,
                            child: Text(e[widget.labelAlias] ?? ''),
                          ),
                        )
                        .toList(),
                    Container(
                      height: _height,
                      alignment: Alignment.center,
                    ),
                    Container(
                      height: _height,
                      alignment: Alignment.center,
                    ),
                  ],
                ),
              ),
            ),
          ),
          Container(
            color: Colors.transparent,
            child: Column(
              children: [
                Opacity(
                  opacity: 0.6,
                  child: Container(
                    height: _height,
                    color: Color(0xffffffff),
                  ),
                ),
                Opacity(
                  opacity: 0.4,
                  child: Container(
                    height: _height,
                    color: Color(0xffffffff),
                  ),
                ),
                Container(
                  height: _height - 2,
                  decoration: BoxDecoration(
                    color: Colors.transparent,
                    border: Border(
                      top: BorderSide(
                        color: Color(0xffe3e3e3),
                        width: 1,
                      ),
                      bottom: BorderSide(
                        color: Color(0xffe3e3e3),
                        width: 1,
                      ),
                    ),
                  ),
                ),
                Opacity(
                  opacity: 0.4,
                  child: Container(
                    height: _height,
                    color: Color(0xffffffff),
                  ),
                ),
                Opacity(
                  opacity: 0.6,
                  child: Container(
                    height: _height,
                    color: Color(0xffffffff),
                  ),
                ),
              ],
            ),
          ),
          Container(
            height: _height * 5,
            child: NotificationListener<ScrollNotification>(
              onNotification: (ScrollNotification notification) {
                double progress =
                    notification.metrics.pixels / ((_length + 4) * _height);
                if (notification is ScrollUpdateNotification) {
                  _overallMoveController.value = progress;
                } else if (notification is ScrollEndNotification) {
                  _handleDragEnd(progress);
                }

                return true; //放开此行注释后，进度条将失效
              },
              child: ListView(
                controller: _controller,
                children: List.generate(
                  _length + 4,
                  (index) => Container(
                    color: Colors.transparent,
                    height: _height,
                  ),
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}
